/**********************************************************************************************/
/* FBL_ByteSwap.h																		 	  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                        					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __FBL_ByteSwap_h 
	#define __FBL_ByteSwap_h

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
// Swap in place:
inline void FBL_Swap_2_bytes( short* inValue )
{
	char* p = (char*) inValue;

	char p0 = p[0];

	p[0]	 = p[1];
	p[1]	 = p0;
}

/**********************************************************************************************/
// Swap by copy:
inline void FBL_Swap_2_bytes( const short* inValue, short* outValue )
{
	CHECK_FBL( inValue != NULL ); 

	char* p = (char*) inValue;
	char* d = (char*) outValue;

	d[1]	= p[0];
	d[0]	= p[1];
}


/**********************************************************************************************/
// Swap in place:
inline void FBL_Swap_4_bytes( long* inValue )
{
	CHECK_FBL( inValue != NULL ); 

	char* p = (char*) inValue;

	// swap 0 and 3 bytes:
	char p0 = p[0];
	char p1 = p[1];
	
	p[0] = p[3];
	p[1] = p[2];
	p[2] = p1;
	p[3] = p0;

}

/**********************************************************************************************/
// Swap by copy:
inline void FBL_Swap_4_bytes( const long* inValue, long* outValue )
{
	CHECK_FBL( inValue != NULL ); 

	char* p = (char*) inValue;
	char* d = (char*) outValue;

	d[0] = p[3];
	d[1] = p[2];
	d[2] = p[1];
	d[3] = p[0];
}


/**********************************************************************************************/
// Swap in place:
inline void FBL_Swap_8_bytes( double* inValue )
{
	CHECK_FBL( inValue != NULL ); 

	char* p = (char*) inValue;

	// swap 0 and 3 bytes:
	char p0 = p[0];
	char p1 = p[1];
	char p2 = p[2];
	char p3 = p[3];
	
	p[0] = p[7];
	p[1] = p[6];
	p[2] = p[5];
	p[3] = p[4];
	p[4] = p3;
	p[5] = p2;
	p[6] = p1;
	p[7] = p0;

}

/**********************************************************************************************/
// Swap by copy:
inline void FBL_Swap_8_bytes( const double* inValue, double* outValue )
{
	CHECK_FBL( inValue != NULL ); 

	char* p = (char*) inValue;
	char* d = (char*) outValue;

	d[0] = p[7];
	d[1] = p[6];
	d[2] = p[5];
	d[3] = p[4];
	d[4] = p[3];
	d[5] = p[2];
	d[6] = p[1];
	d[7] = p[0];
}


/**********************************************************************************************/
// Swap in place:
/*inline void FBL_Swap_10_bytes( ldouble* inValue )
{
	CHECK_FBL( inValue != NULL ); 

	char* p = (char*) inValue;

	// swap 0 and 3 bytes:
	char p0 = p[0];
	char p1 = p[1];
	char p2 = p[2];
	char p3 = p[3];
	char p4 = p[4];
	
	p[0] = p[9];
	p[1] = p[8];
	p[2] = p[7];
	p[3] = p[6];
	p[4] = p[5];
	
	p[5] = p4;
	p[6] = p3;
	p[7] = p2;
	p[8] = p1;
	p[9] = p0;
}*/

/**********************************************************************************************/
// Swap by copy:
/*inline void FBL_Swap_10_bytes( const ldouble* inValue, ldouble* outValue )
{
	CHECK_FBL( inValue != NULL ); 

	char* p = (char*) inValue;
	char* d = (char*) outValue;

	d[0] = p[9];
	d[1] = p[8];
	d[2] = p[7];
	d[3] = p[6];
	d[4] = p[5];
	d[5] = p[4];
	d[6] = p[3];
	d[7] = p[2];
	d[8] = p[1];
	d[9] = p[0];
}*/


#pragma mark -


/**********************************************************************************************/
// Swap by copy:
// in fact for Medium we return 3 bytes in Windows byte order.
inline void* FBL_Long_Medium( const long* inValue, void* outValue )
{
	char* p = (char*) inValue;
	char* d = (char*) outValue;

#if FBL_BIG_ENDIAN	
	d[0] = p[3];
	d[1] = p[2];
	d[2] = p[1];
#else
	d[0] = p[0];
	d[1] = p[1];
	d[2] = p[2];
#endif // FBL_BIG_ENDIAN

	return outValue;
}


/**********************************************************************************************/
inline void* FBL_Long_UMedium( const long* inValue, void* outValue )
{
	return FBL_Long_Medium( inValue, outValue );
}


/**********************************************************************************************/
// Swap by copy:
// in fact for Medium we return 3 bytes in Winndows byte order.
inline long* FBL_Medium_Long( const void* inValue, long* outValue )
{
	char* p = (char*) inValue;
	char* d = (char*) outValue;

#if FBL_BIG_ENDIAN	

	if( p[2] >= 0x00 )
			d[0] = (char) 0x00;	// positive
	else	d[0] = (char) 0xFF;	// negative

	d[1] = p[2];
	d[2] = p[1];
	d[3] = p[0];

#else

	d[0] = p[0];
	d[1] = p[1];
	d[2] = p[2];

	if( p[2] >= 0x00 )
			d[3] = (char) 0x00;	// positive
	else	d[3] = (char) 0xFF;	// negative
	
#endif // FBL_BIG_ENDIAN

	return outValue;
}


/**********************************************************************************************/
// Swap by copy:
// in fact for Medium we return 3 bytes in Winndows byte order.
inline long* FBL_UMedium_Long( const void* inValue, long* outValue )
{
	char* p = (char*) inValue;
	char* d = (char*) outValue;

#if FBL_BIG_ENDIAN	

	d[0] = 0;
	d[1] = p[2];
	d[2] = p[1];
	d[3] = p[0];

#else

	d[0] = p[0];
	d[1] = p[1];
	d[2] = p[2];
	d[3] = 0;

#endif // FBL_BIG_ENDIAN

	return outValue;
}


#pragma mark -


/**********************************************************************************************/
// Swap of array of Longs in place:
inline void FBL_Swap_2_bytesArray( short* inArray, ulong inCount )
{
	CHECK_FBL( inArray != NULL ); 

	for( ulong i = 0; i < inCount; ++i )
		FBL_Swap_2_bytes( inArray++ );
}


/**********************************************************************************************/
// Swap of array of Longs by copy:
inline void FBL_Swap_2_bytesArray( const short* inArray, short* inDest, ulong inCount )
{
	CHECK_FBL( inArray != NULL ); 

	for( ulong i = 0; i < inCount; ++i )
		FBL_Swap_2_bytes( inArray++, inDest++ );
}

	
/**********************************************************************************************/
// Swap of array of Longs in place:
inline void FBL_Swap_4_bytesArray( long* inArray, ulong inCount )
{
	CHECK_FBL( inArray != NULL ); 

	for( ulong i = 0; i < inCount; ++i )
		FBL_Swap_4_bytes( inArray++ );
}


/**********************************************************************************************/
// Swap of array of Longs by copy:
inline void FBL_Swap_4_bytesArray( const long* inArray, long* inDest, ulong inCount )
{
	CHECK_FBL( inArray != NULL ); 

	for( ulong i = 0; i < inCount; ++i )
		FBL_Swap_4_bytes( inArray++, inDest++ );
}

	
/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __FBL_ByteSwap_h

