/**********************************************************************************************/
/* VDK_Field.h																				  */
/*                                                                       					  */
/* Valentina SDK    																		  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/

#ifndef __VDK_Field_h 
	#define __VDK_Field_h
	
#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL_String.h>
#	include <VSDK/FBL_Encryption.h>
#else
#	include <FBL_String.h>
#	include <FBL_Encryption.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif

/**********************************************************************************************/
// internal classes:
class  FBL_BaseField;

// opened classes:
class FBL_Task;
class FBL_Set;
class FBL_BitSet;
class FBL_ArraySet;
class FBL_Formula;
class FBL_ConstFormula;

// VDK classes:
class VDK_Field;


/**********************************************************************************************/
VDK_Field* VDK_NewField( const char* inName, uchar inType );
// Create C++ object of VDK_Field 


/**********************************************************************************************/
class FBL_EXP VDK_Field 
{
		friend class VDK_DataBase;
		friend class VDK_BaseObject;
		friend class VDK_Cursor;

	public://///////////////////////////////////////////////////////////////////////////////////
 
 		/* Construction/destruction: */
 
FBL_EXP							VDK_Field( const char* inName, uchar inType, ushort inFlags = 0 );								
FBL_EXP virtual 				~VDK_Field( void );

FBL_EXP	const char*				GetMethod( void ) const;

								// If this field must be calculated then you must specify formula
								// for it at least empty string BEFORE Create a database on the disk.
FBL_EXP	void					SetMethod( const char* inStr );
		
virtual	VDK_Field*				Clone( void ) { return NULL; }


		/* Encryption methods: */

FBL_EXP	bool					IsEncrypted( void ) const;		
		
FBL_EXP	void					SetEncryption( 
									FBL_EncryptionKeyCPtr inKey );
									
FBL_EXP	void					ChangeEncryption( 
									FBL_EncryptionKeyCPtr inOldKey,
									FBL_EncryptionKeyCPtr inNewKey );


		/* Flag methods: */		
						
FBL_EXP bool					CheckFlags( ushort inFlags ) const;
FBL_EXP void					SetFlags( ushort inFlags );
FBL_EXP void					ClearFlags( ushort inFlags );


		/* Accessories: */
		
		uchar					GetType ( void ) const	{ return mType; }
FBL_EXP	virtual	ulong			GetBytes ( void ) const; 
							
		VDK_BaseObject*			GetBaseObject( void ) const { return mBaseObject; }
		
FBL_EXP	virtual	const char*		GetName( void ) const;
FBL_EXP	virtual	void			SetName( const char* inNewName );
		
								// Fill by zero memory buffer of the field and 
								// set flag mIsNull of field to the true.					 
FBL_EXP	virtual	void			SetBlank( void );
		
								// Returns true if the field doesn't have value.
								// If the field is not Nullable then always returns false.
FBL_EXP	virtual	bool			IsNull( void ) const;

								// Retruns value of a field as String. Work for Text field also.
								// You must destroy FBL_String object. 		
FBL_EXP virtual	FBL_String*		GetString( ulong inLengthLimit = (ulong) -1 );

								// Store value of field into outString, adds zero at end.
								// Returns pointer after end of string (i.e. on zero). 		
FBL_EXP virtual	char*			GetString( char* outString, ushort inBufferBytes = (ushort) -1 );

								// This methods at first prepare gDTFormat, 
								// then call the bottleneck methods ::FBL_GetString() and ::FBL_SetString()
FBL_EXP	virtual void 			SetString( const char* inStr );
														

		/* Search methods */

								// Returns a result of search in range {LeftValue,RightValue}.
								// You must specify if values will be included or exluded from range.
FBL_EXP	virtual	FBL_BitSet*		Find( const void* inLeftValue , bool inLeftInclude, 
									  const void* inRightValue, bool inRightInclude );
	
								// Returns a result of search given inKey.
								// if pKey is NULL use for search current value of field
FBL_EXP	virtual FBL_BitSet* 	Find( const void* inKey = NULL );
						
								// Return true if find current field's value in its Index.
								// If there is outCount ptr then return count of found value.
FBL_EXP	virtual	bool			FindValue( ulong* outCount = NULL );

								// Returns a result of search in range {LeftValue,RightValue}
								// as FBL_ArraySet. Useful if its known that set will not big.
								// You must specify if values will be included or exluded from range.
FBL_EXP	virtual	FBL_ArraySet*	FindAsArraySet( void* inLeftValue , bool inLeftInclude, 
											  void* inRightValue, bool inRightInclude );

								// Returns records which have NULL value for this field. 
FBL_EXP	virtual	FBL_BitSet* 	FindNulls( void );
							
								// Returns records which have NOT NULL value for this field. 
FBL_EXP	virtual	FBL_BitSet* 	FindNotNulls( void );
				
								
		/* Task methods: */
		
FBL_EXP virtual	FBL_Task*		MakeApplyFormula(
									FBL_Formula*		inFormula,	
									FBL_Set*	 		inSelection,
									bool				inOwnsSelection = false,
									const char*			inTaskName = NULL );

FBL_EXP virtual	FBL_Task*		MakeUseFormula(
									FBL_ConstFormula*	inFormula,	
									FBL_Set*	 		inSelection,
									bool				inOwnsSelection = false,
									const char*			inTaskName = NULL );
	

	public://////////////////////////////////////////////////////////////////////////////////

FBL_EXP virtual void			FinishCreateSelf( void );

								// Returns direct pointer on the place in memory of current value
								// of the field. Size of this buffer is GetBytes().
								// Be careful using this.  :-)
FBL_EXP virtual char* 			GetValuePtr ( bool inDoMethod = false ) const;
		
FBL_EXP FBL_REC_ID				GetRecID( void ) const;
	
								// if you have set value of field by GetValuePtr() then you need
								// set this flag to notify VSDK that value was changed. 					
FBL_EXP	virtual void			SetIsNull( bool inIsNull );	
		
FBL_EXP	void					DoMethod( FBL_REC_ID inRecID, void* outResult );
								
		// internal methods:

		FBL_BaseField*			GetImplementation( void )	{ return mFieldImp; }
		bool					IsInited( void ) const		{ return (mFieldImp != NULL); }

		
	protected://////////////////////////////////////////////////////////////////////////////////

		FBL_BaseField*			mFieldImp; 		// pointer on the object of kernel
		VDK_BaseObject*			mBaseObject;	// its BaseObject(Table)

		// data members which store information about this field to send it to the kernel
		// on creation of database. Used only on creation. 
		// When kernel objects exists all information come from there.
			
		FBL_String 				mName;			// name of this field.
		ushort					mFlags;			// fIndexed, fUnique, ...
		uchar					mType;			// type of the field.
		FBL_String* 			mMethod;


	public://///////////////////////////////////////////////////////////////////////////////////
 
		// These 2 data members allows easy to use VSDK to develop plugins
		// such as Valentina for REALbasic, Valentina for Java, which do have garbage collection.
			
			// An owner should set up this back reference, so when destructor of this class
			// will be called FROM kernel it will set back ref to NULL via mBackRef. 
			// The owner will be able to handle correctly this situation and won't try access
			// already destroyed kernel object.
		VDK_Field**				mBackRef;
				
			// here you can put pointer on your own additional data (or even structure)
			// if you need attach several variables to this C++ instance.	
		void*					mXtraData;	

			// Destructor ~VDK_Field() will call this pointer if you set it.
		void					(*mpFinalize)(VDK_Field* inField );
}; 

/**********************************************************************************************/
#endif  

